@@ -1,4 +1,4 @@
-
+#
 #    make_pkg.py
 #
 import os
@@ -10,6 +10,14 @@ import wb_version
 import pysvn
 import time
 
+def os_system( cmd ):
+    print 'Info: %s' % cmd
+    sys.stdout.flush()
+    if os.system( '%s 2>&1' % cmd ) == 0:
+        return
+    print 'Error: command failed'
+    sys.exit( 1 )
+
 pymaj, pymin, pypat, _, _ = sys.version_info
 wb_version_string = '%d.%d.%d-%d' % (wb_version.major, wb_version.minor, wb_version.patch, wb_version.build)
 python_version_string = '%d.%d.%d' % (pymaj, pymin, pypat)
@@ -26,7 +34,7 @@ build_time_str = time.strftime( '%d-%b-%Y %H:%M', time.localtime( build_time ) )
 tmpdir = os.path.join( os.getcwd(), 'tmp' )
 if os.path.exists( tmpdir ):
     print 'Info: Clean up tmp directory'
-    os.system( 'rm -rf tmp' )
+    os_system( 'rm -rf tmp' )
 
 print 'Info: Create directories'
 
@@ -48,18 +56,25 @@ for cp_src, cp_dst_dir_fmt in [
     if os.path.exists( cp_src ):
         cmd = 'cp -f %s tmp/%s' % (cp_src, cp_dst_dir_fmt % locals())
         print 'Info: %s' % cmd
-        os.system( cmd )
+        os_system( cmd )
     else:
         print 'Error: cannot find %s' % cp_src
         sys.exit( 1 )
 
+print 'Info: Export WorkBench_files'
+if os.path.exists( '../../Docs/WorkBench_files/.svn' ):
+    os_system( 'svn export ../../Docs/WorkBench_files tmp/%s/WorkBench_files' % pkg_filename )
+else:
+    os_system( 'mkdir tmp/%s/WorkBench_files' % pkg_filename )
+    os_system( 'cp ../../Docs/WorkBench_files/* tmp/%s/WorkBench_files' % pkg_filename )
+
 print 'Info: Create tmp/ReadMe.html'
 f = file('tmp/ReadMe.html','w')
 f.write('''<html>
 <body>
 <h1>pysvn WorkBench %(wb_version_string)s for Mac OS X and Subversion %(svn_version_string)s</h1>
 
-<h2>Copyright Barry A. Scott (c) 2003-2005</h2>
+<h2>Copyright Barry A. Scott (c) 2003-2006</h2>
 
 <h2>Mail <a href="mailto:barry@barrys-emacs.org">barry@barrys-emacs.org</a></h2>
 
@@ -72,12 +87,9 @@ f.write('''<html>
 f.close()
 
 print 'Info: python bundle'
-rc = os.system( '/usr/bin/python make_wb_bundle.py tmp/%s' % pkg_filename)
-if rc != 0:
-    print 'Error: make_wb_bundle.py failed'
-    sys.exit( 1 )
+os_system( '/usr/bin/python make_wb_bundle.py tmp/%s' % pkg_filename)
 
 print 'Info: Make Disk Image'
-os.system( 'hdiutil create -srcfolder tmp/%s tmp/tmp.dmg' % pkg_filename )
-os.system( 'hdiutil convert tmp/tmp.dmg -format UDZO -imagekey zlib-level=9 ' 
+os_system( 'hdiutil create -srcfolder tmp/%s tmp/tmp.dmg' % pkg_filename )
+os_system( 'hdiutil convert tmp/tmp.dmg -format UDZO -imagekey zlib-level=9 ' 
         '-o tmp/%s.dmg' % pkg_filename )
