@@ -3,6 +3,12 @@
 #
 import bundlebuilder
 import sys
+
+# make sure that we get 2.6 and not an earlier version
+if not hasattr(sys, 'frozen'):
+    import wxversion
+    wxversion.select('2.6')
+
 import wx
 import os
 import traceback
@@ -54,6 +60,22 @@ try:
         if not lib_found:
             raise ValueError( 'Cannot find lib %s' % libname )
 
+    # add in libs from Fink
+    if os.path.exists( '/sw' ):
+        for libname in [
+                "/sw/lib/libintl.3.dylib",
+                "/sw/lib/libiconv.2.dylib",
+                ]:
+            print 'Info: Manually adding lib %s' % libname
+            myapp.libs.append( libname )
+    if os.path.exists( '/opt/local' ):
+        for libname in [
+                "/opt/local/lib/libapr-1.0.dylib",
+                "/opt/local/lib/libz.1.dylib",
+                ]:
+            print 'Info: Manually adding lib %s' % libname
+            myapp.libs.append( libname )
+
     # Here we build the app!
     myapp.setup()
     myapp.build()
