@@ -9,4 +9,4 @@ fi
 
 export LD_LIBRARY_PATH=${SVNCPP_LIB}
 export DYLD_LIBRARY_PATH=${SVNCPP_LIB}
-pythonw wb_main.py $*
+pythonw$1 wb_main.py $*
