@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -15,6 +15,12 @@
 VERSION_STRING = "Uncontrolled"
 
 import sys
+
+# make sure that we get 2.6 and not an earlier version
+if not hasattr(sys, 'frozen'):
+    import wxversion
+    wxversion.select('2.6')
+
 import wx
 import wb_diff_frame
 import wb_preferences
