@@ -11,7 +11,6 @@
 
 '''
 import sys
-import os
 import wx
 import wx.stc
 import logging
@@ -47,7 +46,7 @@ class WbFrame(wx.Frame):
     status_progress = 1
     status_action = 2
     status_num_fields = 3
-    status_widths = [-1,100,-2]
+    status_widths = [-1, 100, -2]
 
     def __init__( self, app ):
         self.app = app
@@ -83,8 +82,7 @@ class WbFrame(wx.Frame):
         self.menu_actions.Append(  wb_ids.id_File_Browser, '&File Browser', 'File Browser' )
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append(  wb_ids.id_File_Edit, 'Edit', 'Edit' )
-        if wx.Platform in ['__WXMSW__','__WXMAC__']:
-            self.menu_actions.Append(  wb_ids.id_Shell_Open, 'Open', 'Open' )
+        self.menu_actions.Append(  wb_ids.id_Shell_Open, 'Open', 'Open' )
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append(  wb_ids.id_SP_DiffWorkBase, 'Diff WC vs. BASE...', 'Diff WC vs. BASE...' )
         self.menu_actions.Append(  wb_ids.id_SP_DiffWorkHead, 'Diff WC vs. HEAD...', 'Diff WC vs. HEAD...' )
@@ -181,7 +179,7 @@ class WbFrame(wx.Frame):
         t = self.CreateToolBar( name="main",
                                 style=wx.TB_HORIZONTAL ) # | wx.NO_BORDER | wx.TB_TEXT )
 
-        bitmap_size = (32,32)
+        bitmap_size = (32, 32)
         t.SetToolBitmapSize( bitmap_size )
         t.AddSimpleTool( wb_ids.id_SP_EditCut,
             wb_images.getBitmap( 'toolbar_images/editcut.png', bitmap_size ),
@@ -203,10 +201,9 @@ class WbFrame(wx.Frame):
         t.AddSimpleTool( wb_ids.id_File_Edit,
             wb_images.getBitmap( 'toolbar_images/edit.png', bitmap_size ),
             'Edit File', 'Edit File' )
-        if wx.Platform in ['__WXMSW__','__WXMAC__']:
-            t.AddSimpleTool( wb_ids.id_Shell_Open,
-                wb_images.getBitmap( 'toolbar_images/open.png', bitmap_size ),
-                'Open File', 'Open File' )
+        t.AddSimpleTool( wb_ids.id_Shell_Open,
+            wb_images.getBitmap( 'toolbar_images/open.png', bitmap_size ),
+            'Open File', 'Open File' )
         t.AddSeparator()
         t.AddSimpleTool( wb_ids.id_SP_DiffWorkBase,
             wb_images.getBitmap( 'toolbar_images/diff.png', bitmap_size ),
@@ -453,7 +450,7 @@ class WbFrame(wx.Frame):
 
     # Handler for the About menu command
     def OnCmdAbout(self, event):
-        strMessage =    ('Work Bench version: %d.%d.%d-%d\n' %
+        str_message =    ('Work Bench version: %d.%d.%d-%d\n' %
                     (wb_version.major, wb_version.minor,
                      wb_version.patch, wb_version.build) +
                 '\n' + wb_source_control_providers.getProviderAboutStrings() +
@@ -461,7 +458,7 @@ class WbFrame(wx.Frame):
                 '\nPython %d.%d.%d %s %d\n' % sys.version_info +
                 '\nCopyright Barry Scott (c) 2003-2006. All rights reserved'
                 )
-        wx.LogMessage( strMessage )
+        wx.LogMessage( str_message )
 
     def OnCmdPreferences( self, event ):
         pref_dialog = wb_preferences_dialog.PreferencesDialog( self, self.app )
@@ -491,26 +488,26 @@ class WbFrame(wx.Frame):
         event.Skip()
 
     def OnHorizSashPositionChanged( self, event ):
-        w, h = self.h_split.GetClientSizeTuple()
+        _, h = self.h_split.GetClientSizeTuple()
         win_prefs = self.app.prefs.getWindow()
         win_prefs.h_sash_ratio = float( event.GetSashPosition() ) / float( h )
         event.Skip()
 
     def OnVertSashPositionChanged( self, event ):
-        w, h = self.v_split.GetClientSizeTuple()
+        w, _ = self.v_split.GetClientSizeTuple()
         win_prefs = self.app.prefs.getWindow()
         win_prefs.v_sash_ratio = float( event.GetSashPosition() ) / float( w )
         event.Skip()
 
     def OnHorizSize( self, event ):
         win_prefs = self.app.prefs.getWindow()
-        w, h = self.h_split.GetClientSizeTuple()
+        _, h = self.h_split.GetClientSizeTuple()
         self.h_split.SetSashPosition( max( 200, int( h * win_prefs.h_sash_ratio ) ) )
         event.Skip()
 
     def OnVertSize( self, event ):
         win_prefs = self.app.prefs.getWindow()
-        w, h = self.v_split.GetClientSizeTuple()
+        w, _ = self.v_split.GetClientSizeTuple()
         self.v_split.SetSashPosition( max( 200, int( w * win_prefs.v_sash_ratio ) ) )
         event.Skip()
 
@@ -622,9 +619,18 @@ class WbFrame(wx.Frame):
             bookmark_list.append( bm_prefs.getBookmark( bm_name ) )
 
         def sortFunction( a, b ):
-            rc = cmp( a.menu_folder.lower(), b.menu_folder.lower() )
-            if rc == 0:
-                rc = cmp( a.menu_name.lower(), b.menu_name.lower() )
+            #
+            #   Sort only by name - do not sort folder then name
+            #
+            a_name = a.menu_folder
+            if a_name == '':
+                a_name = a.menu_name
+
+            b_name = b.menu_folder
+            if b_name == '':
+                b_name = b.menu_name
+
+            rc = cmp( a_name.lower(), b_name.lower() )
             return rc
 
         bookmark_list.sort( sortFunction )
@@ -804,23 +810,29 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpDiffWorkBase( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_list.modified )
+        if self.ui_state_list is self.ui_state_focus:
+            event.Enable( self.ui_state_list.modified )
+        else:
+            event.Enable( True )
 
     def OnSpDiffWorkHead( self, event ):
         return self.Sp_Dispatch( 'OnSpDiffWorkHead' )
 
-    def OnSpDiffWorkBranchOriginBase( self, event ):
-        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginBase' )
-
-    def OnSpDiffWorkBranchOriginHead( self, event ):
-        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginHead' )
-
     def OnUpdateUiSpDiffWorkHead( self, event ):
         self.getUpdateUiState()
         if self.ui_state_focus.need_checkout:
             event.Enable( False )
         else:
-            event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
+            if self.ui_state_list is self.ui_state_focus:
+                event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
+            else:
+                event.Enable( self.ui_state_tree.versioned )
+
+    def OnSpDiffWorkBranchOriginBase( self, event ):
+        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginBase' )
+
+    def OnSpDiffWorkBranchOriginHead( self, event ):
+        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginHead' )
 
     def OnUpdateUiSpDiffWorkBranchOriginBase( self, event ):
         self.getUpdateUiState()
@@ -976,12 +988,12 @@ class WbFrame(wx.Frame):
         self.clearUpdateUiState()
 
         if self.event_handler is None:
-            print 'No event_handler, cannot call',sp_func_name
+            print 'No event_handler, cannot call', sp_func_name
             return None
 
         fn = getattr( self.event_handler, sp_func_name, None )
         if fn is None:
-            print 'Not implemented:',sp_func_name
+            print 'Not implemented: %r in %r' % (sp_func_name, self.event_handler)
             return None
         else:
             return fn()
@@ -992,14 +1004,14 @@ class WbFrame(wx.Frame):
             self.ui_state_tree = self.tree_panel.getUpdateUiState()
             if self.ui_state_tree is None:
                 self.ui_state_tree = wb_tree_panel.TreeState()
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree need_checkout',self.ui_state_tree.need_checkout
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree need_checkout', self.ui_state_tree.need_checkout
 
         if self.ui_state_list is None:
             if wb_config.debug_selection: print 'Z: getUpdateUiState() list'
             self.ui_state_list = self.list_panel.getUpdateUiState()
             if self.ui_state_list is None:
                 self.ui_state_list = wb_list_panel_common.ListItemState()
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() list need_checkout',self.ui_state_list.need_checkout
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() list need_checkout', self.ui_state_list.need_checkout
 
         if self.ui_state_focus is None:
             if self.event_handler is None:
@@ -1011,7 +1023,7 @@ class WbFrame(wx.Frame):
             else:
                 if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is List set list'
                 self.ui_state_focus = self.ui_state_list
-            if wb_config.debug_selection: print 'Z: getUpdateUiState() focus need_checkout',self.ui_state_focus.need_checkout
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() focus need_checkout', self.ui_state_focus.need_checkout
 
     def clearUpdateUiState( self ):
         if wb_config.debug_selection: print 'Z: clearUpdateUiState()'
@@ -1095,7 +1107,7 @@ class LogHandler(logging.Handler):
                 self.log_ctrl.WriteDebug( msg )
             else:
                 self.log_ctrl.WriteError( msg )
-        except:
+        except Exception:
             self.handleError(record)
 
 #--------------------------------------------------------------------------------
@@ -1121,7 +1133,7 @@ class StyledLogCtrl(wx.stc.StyledTextCtrl):
         self.SetMarginWidth(2, 0)
 
         self.StyleSetSpec( wx.stc.STC_STYLE_DEFAULT, 
-                "size:%d,face:%s,fore:#000000" % (point_size,face) )
+                "size:%d,face:%s,fore:#000000" % (point_size, face) )
 
         self.StyleSetSpec( self.style_normal,   "fore:#000000" )
         self.StyleSetSpec( self.style_error,    "fore:#DC143C" )    # Crimson
@@ -1137,7 +1149,7 @@ class StyledLogCtrl(wx.stc.StyledTextCtrl):
     def WriteStyledText( self, text, style ):
         # only allowed to use GUI objects on the foreground thread
         if not self.app.isMainThread():
-            self.app.foregroundProcess( self.WriteStyledText, (text,style) )
+            self.app.foregroundProcess( self.WriteStyledText, (text, style) )
             return
 
         self.SetReadOnly(False)
