@@ -492,8 +492,5 @@ class ListHandler:
     def __init__( self, list_panel ):
         self.list_panel = list_panel
 
-    def initList( self, sort_data ):
-        raise wb_exceptions.InternalError( 'initList not implemented' )
-
     def sortList( self, sort_data ):
         raise wb_exceptions.InternalError( 'sortList not implemented' )
