@@ -17,24 +17,22 @@ import locale
 
 # help debug when stdout goes nowhere useful
 # Mac OS X and Windows are the main problems
-if os.environ.get( 'PYSVN_WORKBENCH_STDOUT_LOG', None ) or sys.platform == 'darwin':
+if( os.environ.get( 'PYSVN_WORKBENCH_STDOUT_LOG', None )
+or sys.platform == 'darwin' ):
     try:
-        sys.stdout = file( os.environ.get( 'PYSVN_WORKBENCH_STDOUT_LOG', '/tmp/workbench.tmp' ),'w')
+        sys.stdout = file( os.environ.get( 'PYSVN_WORKBENCH_STDOUT_LOG', '/tmp/workbench.tmp' ), 'w' )
         sys.stderr = sys.stdout
         print 'PySVN WorkBench starting'
         sys.stdout.flush()
-    except:
+    except EnvironmentError:
         pass
 
 # make sure that we get 2.6 and not an earlier version
-if not hasattr(sys, 'frozen'):
-    try:
-        import wxversion
-        wxversion.select('2.6')
-    except:
-        pass
-
-import wx
+try:
+    import wxversion
+    wxversion.select('2.6')
+except:
+    pass
 
 import wb_app
 import wb_subversion_provider
@@ -43,6 +41,7 @@ def prerequesitChecks():
     return 1
 
 def main( args ):
+    startup_dir = os.getcwd()
     if 1:
         if sys.platform == 'win32':
             os.chdir( os.environ['USERPROFILE'] )
@@ -53,7 +52,7 @@ def main( args ):
 
     # don't pollute any subprocesses with env vars
     # from packaging processing
-    for envvar in ['PYTHONPATH','PYTHONHOME','PYTHONEXECUTABLE']:
+    for envvar in ['PYTHONPATH', 'PYTHONHOME', 'PYTHONEXECUTABLE']:
         if os.environ.has_key( envvar ):
             del os.environ[ envvar ]
 
@@ -64,7 +63,7 @@ def main( args ):
     wb_subversion_provider.registerProvider()
 
     # Create the win application and start its message loop
-    app = wb_app.WbApp( args )
+    app = wb_app.WbApp( startup_dir, args )
 
     if not prerequesitChecks():
         return 1
