@@ -35,6 +35,7 @@ class Preferences:
         self.pref_handlers['View'] = ViewPreferences( self.app )
         self.pref_handlers['Editor'] = EditorPreferences( self.app )
         self.pref_handlers['Shell'] = ShellPreferences( self.app )
+        self.pref_handlers['DiffTool'] = DiffToolPreferences( self.app )
 
         # read preferences into the handlers
         self.readPreferences()
@@ -485,3 +486,42 @@ class ShellPreferences(PreferenceSection):
             set_option.set( 'terminal', self.shell_terminal )
         if self.shell_file_browser:
             set_option.set( 'file_browser', self.shell_file_browser )
+
+class DiffToolPreferences(PreferenceSection):
+    def __init__( self, app ):
+        PreferenceSection.__init__( self, 'DiffTool' )
+        self.app = app
+
+        self.diff_tool_mode = 'built-in'
+        self.gui_diff_tool = ''
+        self.shell_diff_tool = ''
+        self.gui_diff_tool_options = ''
+        self.shell_diff_tool_options = ''
+
+    def readPreferences( self, pref_data ):
+        get_option = GetOption( pref_data, self.section_name )
+
+        if get_option.has( 'diff_tool_mode' ):
+            self.diff_tool_mode = get_option.getstr( 'diff_tool_mode' )
+        if get_option.has( 'diff_tool' ):
+            self.gui_diff_tool = get_option.getstr( 'diff_tool' )
+        if get_option.has( 'shell_diff_tool' ):
+            self.shell_diff_tool = get_option.getstr( 'shell_diff_tool' )
+        if get_option.has( 'diff_tool_options' ):
+            self.gui_diff_tool_options = get_option.getstr( 'diff_tool_options' )
+        if get_option.has( 'shell_diff_tool_options' ):
+            self.shell_diff_tool_options = get_option.getstr( 'shell_diff_tool_options' )
+
+    def writePreferences( self, pref_data ):
+        set_option = SetOption( pref_data, self.section_name )
+
+        if self.diff_tool_mode != 'built-in':
+            set_option.set( 'diff_tool_mode', self.diff_tool_mode )
+        if self.gui_diff_tool != '':
+            set_option.set( 'diff_tool', self.gui_diff_tool )
+        if self.shell_diff_tool != '':
+            set_option.set( 'shell_diff_tool', self.shell_diff_tool )
+        if self.gui_diff_tool_options != '':
+            set_option.set( 'diff_tool_options', self.gui_diff_tool_options )
+        if self.shell_diff_tool_options != '':
+            set_option.set( 'shell_diff_tool_options', self.shell_diff_tool_options )
