@@ -44,6 +44,7 @@ class PreferencesDialog( wx.Dialog ):
 
         self.pages = []
         self.pages.append( EditorPage( self.notebook, self.app ) )
+        self.pages.append( DiffToolPage( self.notebook, self.app ) )
         self.pages.append( ShellPage( self.notebook, self.app ) )
         self.pages.append( ListColumnsPage( self.notebook, self.app ) )
         # may want view page for editing the control file
@@ -52,9 +53,9 @@ class PreferencesDialog( wx.Dialog ):
         self.notebook.SetAutoLayout( True )
         self.notebook.Layout()
 
-        self.button_ok = wx.Button( self, wx.ID_OK, " OK " )
+        self.button_ok = wx.Button( self, wx.ID_OK, ' OK ' )
         self.button_ok.SetDefault()
-        self.button_cancel = wx.Button( self, wx.ID_CANCEL, " Cancel " )
+        self.button_cancel = wx.Button( self, wx.ID_CANCEL, ' Cancel ' )
 
         self.h_sizer = wx.BoxSizer( wx.HORIZONTAL )
         self.h_sizer.Add( (1, 1), 1, wx.EXPAND )
@@ -104,13 +105,13 @@ class EditorPage(PagePanel):
     def initControls( self ):
         p = self.app.prefs.getEditor()
 
-        self.static_text1 = wx.StaticText( self, -1, "Editor: ", style=wx.ALIGN_RIGHT)
+        self.static_text1 = wx.StaticText( self, -1, 'Editor: ', style=wx.ALIGN_RIGHT)
         self.text_ctrl_editor = wx.TextCtrl( self, -1, p.editor_image, wx.DefaultPosition, wx.Size(415, -1) )
 
-        self.static_text2 = wx.StaticText( self, -1, "Edit Arguments: ", style=wx.ALIGN_RIGHT)
+        self.static_text2 = wx.StaticText( self, -1, 'Edit Arguments: ', style=wx.ALIGN_RIGHT)
         self.text_ctrl_edit_arg = wx.TextCtrl( self, -1, p.editor_options, wx.DefaultPosition, wx.Size(315, -1) )
 
-        self.browse_button = wx.Button( self, -1, " Browse... ")
+        self.browse_button = wx.Button( self, -1, ' Browse... ')
 
         self.grid_sizer = wx.FlexGridSizer( 0, 3, 0, 0 )
         self.grid_sizer.AddGrowableCol( 1 )
@@ -136,16 +137,16 @@ class EditorPage(PagePanel):
 
     def OnBrowseExe( self, event ):
         if wx.Platform == '__WXMSW__':
-            file_type = "Executable files (*.exe)|*.exe"
+            file_type = 'Executable files (*.exe)|*.exe'
         elif wx.Platform == '__WXMAC__':
-            file_type = "Applications|*.app|Executable files|*"
+            file_type = 'Applications|*.app|Executable files|*'
         else:
-            file_type = "Executable files|*"
+            file_type = 'Executable files|*'
 
         filename = self.text_ctrl_editor.GetValue()
         file_dialog = wx.FileDialog( 
             self,
-            "Choose an Executable file",
+            'Choose an Executable file',
             os.path.dirname( filename ),
             os.path.basename( filename ),
             file_type,
@@ -175,14 +176,110 @@ class EditorPage(PagePanel):
 
         if not valid:
             wx.MessageBox(
-                "You must enter a valid editor executable",
-                "Warning",
+                'You must enter a valid editor executable',
+                'Warning',
                 wx.OK | wx.ICON_EXCLAMATION,
                 self )
             return False
 
         return True
 
+
+class DiffToolPage(PagePanel):
+    def __init__( self, notebook, app ):
+        self.app = app
+        PagePanel.__init__( self, notebook, 'Diff Tool' )
+
+    def initControls( self ):
+        p = self.app.prefs.getDiffTool()
+
+        
+        self.mode_values = ['built-in', 'external-gui-diff', 'external-shell-diff', 'svn-diff']
+        self.diff_tools = {'built-in': '', 'external-gui-diff': p.gui_diff_tool, 'external-shell-diff': p.shell_diff_tool, 'svn-diff': ''}
+        self.options = {'built-in': '', 'external-gui-diff': p.gui_diff_tool_options, 'external-shell-diff': p.shell_diff_tool_options, 'svn-diff': ''}
+        self.mode_choice_values = ['Work Bench Diff', 'External GUI Diff Command', 'External Text Diff', 'svn diff']
+        self.mode_choice = wx.Choice( self, -1, choices = self.mode_choice_values )
+        if p.diff_tool_mode not in self.mode_values:
+            p.diff_tool_mode = 'built-in'
+        self.mode = p.diff_tool_mode
+        self.mode_choice.SetSelection( self.mode_values.index( p.diff_tool_mode ) )
+        self.text_ctrl_diff_tool = wx.TextCtrl( self, -1, self.diff_tools[self.mode], wx.DefaultPosition, wx.Size(415, -1) )
+        self.text_ctrl_options = wx.TextCtrl( self, -1, self.options[self.mode], wx.DefaultPosition, wx.Size(315, -1) )
+
+        self.browse_button = wx.Button( self, -1, ' Browse... ')
+
+        self.grid_sizer = wx.FlexGridSizer( 0, 3, 0, 0 )
+        self.grid_sizer.AddGrowableCol( 1 )
+
+        self.grid_sizer.Add( wx.StaticText( self, -1, 'Mode: ', style=wx.ALIGN_RIGHT), 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.mode_choice, 0, wx.ALIGN_LEFT, 5 )
+        self.grid_sizer.Add( (1, 1), 0, wx.EXPAND )
+        
+        self.grid_sizer.Add( wx.StaticText( self, -1, 'Diff Tool: ', style=wx.ALIGN_RIGHT), 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.text_ctrl_diff_tool, 0, wx.EXPAND|wx.ALL, 5 )
+        self.grid_sizer.Add( self.browse_button, 0, wx.EXPAND )
+
+        self.grid_sizer.Add( wx.StaticText( self, -1, 'Tool Arguments: ', style=wx.ALIGN_RIGHT), 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.text_ctrl_options, 0, wx.EXPAND|wx.ALL, 5 )
+        self.grid_sizer.Add( (1, 1), 0, wx.EXPAND )
+
+        self.grid_sizer.Add( wx.StaticText( self, -1, 'Use', style=wx.ALIGN_RIGHT), 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( wx.StaticText( self, -1, '%nl for left file name, %nr for right file name,\n%tl for left title, %tr for right title',
+                                           style=wx.ALIGN_LEFT), 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( (1, 1), 0, wx.EXPAND )
+        
+        self.OnModeChange( None )
+
+        wx.EVT_CHOICE( self, self.mode_choice.GetId(), self.OnModeChange )
+        wx.EVT_BUTTON( self, self.browse_button.GetId(), self.OnBrowseExe )
+
+        return self.grid_sizer
+
+    def savePreferences( self ):
+        self.OnModeChange( None )
+        p = self.app.prefs.getDiffTool()
+        p.diff_tool_mode = self.mode
+        p.gui_diff_tool = self.diff_tools['external-gui-diff']
+        p.shell_diff_tool = self.diff_tools['external-shell-diff']
+        p.gui_diff_tool_options = self.options['external-gui-diff']
+        p.shell_diff_tool_options = self.options['external-shell-diff']
+
+    def OnModeChange( self, event ):
+       self.diff_tools[self.mode] = self.text_ctrl_diff_tool.GetValue()
+       self.options[self.mode] = self.text_ctrl_options.GetValue()
+       self.mode = self.mode_values[self.mode_choice.GetSelection()]
+       self.text_ctrl_diff_tool.SetValue( self.diff_tools[self.mode] )
+       self.text_ctrl_options.SetValue( self.options[self.mode] )
+       external_command = self.mode in ('external-gui-diff', 'external-shell-diff')
+       self.text_ctrl_diff_tool.Enable( external_command )
+       self.browse_button.Enable( external_command )
+       self.text_ctrl_options.Enable( external_command )
+
+    def OnBrowseExe( self, event ):
+        if wx.Platform == '__WXMSW__':
+            file_type = 'Executable files (*.exe)|*.exe'
+        elif wx.Platform == '__WXMAC__':
+            file_type = 'Applications|*.app|Executable files|*'
+        else:
+            file_type = 'Executable files|*'
+
+        filename = self.text_ctrl_diff_tool.GetValue()
+        file_dialog = wx.FileDialog(
+            self,
+            'Choose an Executable file',
+            os.path.dirname( filename ),
+            os.path.basename( filename ),
+            file_type,
+            wx.OPEN )
+
+        if file_dialog.ShowModal() == wx.ID_OK:
+            self.text_ctrl_diff_tool.SetValue( file_dialog.GetPath() )
+
+        file_dialog.Destroy()
+
+    def validate( self ):
+        return True
+
 class ShellPage(PagePanel):
     def __init__( self, notebook, app ):
         self.app = app
@@ -191,7 +288,7 @@ class ShellPage(PagePanel):
     def initControls( self ):
         p = self.app.prefs.getShell()
 
-        self.static_text1 = wx.StaticText( self, -1, "Terminal Init Command: ", style=wx.ALIGN_RIGHT)
+        self.static_text1 = wx.StaticText( self, -1, 'Terminal Init Command: ', style=wx.ALIGN_RIGHT)
         self.text_ctrl_shell_init_command = wx.TextCtrl( self, -1, p.shell_init_command, wx.DefaultPosition, wx.Size(300, -1) )
 
         self.grid_sizer = wx.FlexGridSizer( 0, 2, 0, 0 )
@@ -199,7 +296,7 @@ class ShellPage(PagePanel):
 
         terminal_program_list = wb_shell_commands.getTerminalProgramList()
         if len(terminal_program_list) > 0:
-            self.static_text2 = wx.StaticText( self, -1, "Terminal Program: ", style=wx.ALIGN_RIGHT)
+            self.static_text2 = wx.StaticText( self, -1, 'Terminal Program: ', style=wx.ALIGN_RIGHT)
             self.terminal_program_list_ctrl = wx.Choice( self, -1, choices=terminal_program_list, size=(150,-1) )
             if p.shell_terminal in terminal_program_list:
                 self.terminal_program_list_ctrl.SetStringSelection( p.shell_terminal )
@@ -216,7 +313,7 @@ class ShellPage(PagePanel):
         file_browser_program_list = wb_shell_commands.getFileBrowserProgramList()
 
         if len(file_browser_program_list) > 0:
-            self.static_text3 = wx.StaticText( self, -1, "File Brower Program: ", style=wx.ALIGN_RIGHT)
+            self.static_text3 = wx.StaticText( self, -1, 'File Brower Program: ', style=wx.ALIGN_RIGHT)
             self.file_browser_program_list_ctrl = wx.Choice( self, -1, choices=file_browser_program_list, size=(150,-1) )
             if p.shell_file_browser in file_browser_program_list:
                 self.file_browser_program_list_ctrl.SetStringSelection( p.shell_file_browser )
@@ -259,15 +356,15 @@ class ViewPage(PagePanel):
 
         self.filename_list = wx.ListCtrl( self, ViewPage.id_filename_list, wx.DefaultPosition,
                 wx.Size( -1, 100 ), wx.LC_REPORT )
-        self.filename_list.InsertColumn( 0, "Exclude filename" )
+        self.filename_list.InsertColumn( 0, 'Exclude filename' )
         self.filename_list.SetColumnWidth( 0, 400 )
 
         for index, filename in enumerate( p.excluded_filenames_list ):
             self.filename_list.InsertStringItem( index, filename )
 
-        self.button_add = wx.Button( self, ViewPage.id_add, " Add " )
+        self.button_add = wx.Button( self, ViewPage.id_add, ' Add ' )
         self.button_add.Enable( False )
-        self.button_del = wx.Button( self, ViewPage.id_del, " Delete " )
+        self.button_del = wx.Button( self, ViewPage.id_del, ' Delete ' )
         self.button_del.Enable( False )
         self.pattern_text_ctrl = wx.TextCtrl( self, ViewPage.id_pattern_text, '',
                 wx.DefaultPosition, wx.Size(200, -1), style=wx.TE_PROCESS_ENTER )
@@ -385,16 +482,16 @@ class ListColumnsPage(PagePanel):
 
         self.excluded_list = wx.ListCtrl( self, ListColumnsPage.id_excluded_list, wx.DefaultPosition,
                 wx.Size( 200, 100 ), wx.LC_REPORT )
-        self.excluded_list.InsertColumn( 0, "Column" )
+        self.excluded_list.InsertColumn( 0, 'Column' )
         self.excluded_list.SetColumnWidth( 0, 100 )
-        self.excluded_list.InsertColumn( 1, "Width", wx.LIST_FORMAT_RIGHT )
+        self.excluded_list.InsertColumn( 1, 'Width', wx.LIST_FORMAT_RIGHT )
         self.excluded_list.SetColumnWidth( 1, 80 )
 
         self.included_list = wx.ListCtrl( self, ListColumnsPage.id_included_list, wx.DefaultPosition,
                 wx.Size( 200, 100 ), wx.LC_REPORT )
-        self.included_list.InsertColumn( 0, "Column" )
+        self.included_list.InsertColumn( 0, 'Column' )
         self.included_list.SetColumnWidth( 0, 100 )
-        self.included_list.InsertColumn( 1, "Width", wx.LIST_FORMAT_RIGHT )
+        self.included_list.InsertColumn( 1, 'Width', wx.LIST_FORMAT_RIGHT )
         self.included_list.SetColumnWidth( 1, 80 )
 
         for name in self.column_info.getColumnOrder():
@@ -412,14 +509,14 @@ class ListColumnsPage(PagePanel):
             self.excluded_list.SetStringItem( index, 1, str(info.width) )
 
 
-        self.button_include = wx.Button( self, ListColumnsPage.id_include, " Include --> " )
+        self.button_include = wx.Button( self, ListColumnsPage.id_include, ' Include --> ' )
         self.button_include.Enable( False )
-        self.button_exclude = wx.Button( self, ListColumnsPage.id_exclude, " <-- Exclude " )
+        self.button_exclude = wx.Button( self, ListColumnsPage.id_exclude, ' <-- Exclude ' )
         self.button_exclude.Enable( False )
 
-        self.button_up = wx.Button( self, ListColumnsPage.id_move_up, " Move Up " )
+        self.button_up = wx.Button( self, ListColumnsPage.id_move_up, ' Move Up ' )
         self.button_up.Enable( False )
-        self.button_down = wx.Button( self, ListColumnsPage.id_move_down, " Move Down " )
+        self.button_down = wx.Button( self, ListColumnsPage.id_move_down, ' Move Down ' )
         self.button_down.Enable( False )
 
         self.width_text_ctrl = wx.TextCtrl( self, ListColumnsPage.id_width, '',
