@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -52,8 +52,8 @@ class ProjectDialog(wx.Dialog):
         self.name_label = wx.StaticText( self, -1, 'Project Name:', style=wx.ALIGN_RIGHT )
         self.name_ctrl = wx.TextCtrl( self, -1, '', size=(400,-1) )
 
-        self.g_sizer.Add( self.name_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 3 )
-        self.g_sizer.Add( self.name_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+        self.g_sizer.Add( self.name_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.name_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
         self.g_sizer.Add( (1, 1), 0, wx.EXPAND )
 
         self.wc_path_browse_button = wx.Button( self, wc_path_browse_button_id, " Browse... ")
@@ -61,8 +61,8 @@ class ProjectDialog(wx.Dialog):
         self.wc_path_label = wx.StaticText(self, -1, 'Working copy Path:', style=wx.ALIGN_RIGHT )
         self.wc_path_ctrl = wx.TextCtrl(self, wc_path_text_ctrl_id, '' )
 
-        self.g_sizer.Add( self.wc_path_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 3 )
-        self.g_sizer.Add( self.wc_path_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+        self.g_sizer.Add( self.wc_path_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.wc_path_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
         self.g_sizer.Add( self.wc_path_browse_button, 0, wx.EXPAND )
 
         self.url_trunk_label = wx.StaticText( self, -1, 'Subversion Trunk URL:', style=wx.ALIGN_RIGHT )
@@ -72,14 +72,22 @@ class ProjectDialog(wx.Dialog):
         self.url_tags_ctrl = wx.TextCtrl( self, url_tags_path_text_ctrl_id, '' )
         self.url_tags_enable_ctrl = wx.CheckBox( self, url_tags_path_enable_ctrl_id, 'Enable' )
 
-        self.g_sizer.Add( self.url_trunk_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 3 )
-        self.g_sizer.Add( self.url_trunk_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+        self.g_sizer.Add( self.url_trunk_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.url_trunk_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
         self.g_sizer.Add( (1, 1), 0, wx.EXPAND )
 
-        self.g_sizer.Add( self.url_tags_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 3 )
-        self.g_sizer.Add( self.url_tags_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+        self.g_sizer.Add( self.url_tags_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.url_tags_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
         self.g_sizer.Add( self.url_tags_enable_ctrl, 0, wx.EXPAND )
 
+        self.newfile_dir_label = wx.StaticText( self, -1, "New File Template Folder: ", style=wx.ALIGN_RIGHT)
+        self.newfile_dir_ctrl = wx.TextCtrl( self, -1, '' )
+        self.newfile_dir_browse = wx.Button( self, -1, " Browse... ")
+
+        self.g_sizer.Add( self.newfile_dir_label, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.newfile_dir_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
+        self.g_sizer.Add( self.newfile_dir_browse, 0, wx.EXPAND )
+
         self.button_ok = wx.Button( self, wx.ID_OK, ' OK ' )
         self.button_ok.SetDefault()
         self.button_cancel = wx.Button( self, wx.ID_CANCEL, ' Cancel ' )
@@ -107,6 +115,8 @@ class ProjectDialog(wx.Dialog):
         wx.EVT_TEXT( self, url_trunk_path_text_ctrl_id, self.updateControls )
         wx.EVT_TEXT( self, url_tags_path_text_ctrl_id, self.updateControls )
         wx.EVT_CHECKBOX( self, url_tags_path_enable_ctrl_id, self.updateControls )
+        wx.EVT_BUTTON( self, self.newfile_dir_browse.GetId(), self.OnBrowseNewFileTemplateDir )
+
         self._update_controls_lock = False
 
     def updateControls( self, event=None ):
@@ -199,9 +209,29 @@ class ProjectDialog(wx.Dialog):
 
         self.updateControls()
 
+    def OnBrowseNewFileTemplateDir( self, event ):
+        filename = self.newfile_dir_ctrl.GetValue()
+        if filename == '':
+            filename = self.wc_path_ctrl.GetValue()
+        filename = os.path.expanduser( filename )
+
+        dir_dialog = wx.DirDialog( 
+            self,
+            "Select New File Template directory",
+            filename,
+            style=wx.DD_DEFAULT_STYLE|wx.DD_NEW_DIR_BUTTON )
+
+        if dir_dialog.ShowModal() == wx.ID_OK:
+            self.newfile_dir_ctrl.SetValue( dir_dialog.GetPath() )
+
+        dir_dialog.Destroy()
+
+        self.updateControls()
+
     def getProjectInfo( self ):
         provider = wb_source_control_providers.getProvider( 'subversion' )
         pi = provider.getProjectInfo( self.app )
+        pi.new_file_template_dir = self.newfile_dir_ctrl.GetValue().strip()
 
         pi.init( self.name_ctrl.GetValue().strip(),
             wc_path=os.path.expanduser( self.wc_path_ctrl.GetValue().strip() ),
@@ -223,6 +253,7 @@ class UpdateProjectDialog(ProjectDialog):
         self.wc_path_ctrl.SetValue( self.project_info.wc_path )
         self.url_tags_ctrl.SetValue( self.project_info.tags_url )
         self.url_tags_enable_ctrl.SetValue( self.project_info.tags_url != '' )
+        self.newfile_dir_ctrl.SetValue( self.project_info.new_file_template_dir )
 
         self.updateControls()
 
