@@ -29,7 +29,7 @@ id_include = wx.NewId()
 
 class CheckinFrame(wx.Frame):
     def __init__( self, app, project_info, all_files ):
-        wx.Frame.__init__( self, None, -1, "Check in", size=(700,500) )
+        wx.Frame.__init__( self, None, -1, 'Check in for %s' % project_info.wc_path, size=(700,500) )
 
         self.app = app
 
