@@ -13,11 +13,16 @@
 import wx
 import wx.lib.splitter
 import wx.calendar
-import wb_images
+
+import tempfile
 import time
+import types
+
 import pysvn
+
+import wb_images
 import wb_subversion_utils
-import wb_read_file
+import wb_subversion_diff
 
 id_view_cmd = wx.NewId()
 id_diff_cmd = wx.NewId()
@@ -30,6 +35,70 @@ action_map = {
     'M': 'Modified',
     }
 
+class LogEntry:
+    def __init__( self,
+            rev_number,
+            url,
+            author,
+            date,
+            label,
+            message,
+            changed_paths ):
+        self.rev_number = rev_number
+        self.url = url
+        self.author = author
+        self.date = date
+        self.label = label
+        self.message = message
+        self.changed_paths = changed_paths
+
+def getHistoryEntries( project_info, filename, limit, revision_end ):
+    history_entries = []
+    # need the URL and repos_root_URL
+    # [0] first entry [0][1] the info
+    info = project_info.client_bg.info2( filename, recurse=False )[0][1]
+    if info.repos_root_URL is None:
+        info = project_info.client_bg.info2( info.URL, recurse=False )[0][1]
+
+    log_entries = project_info.client_bg.log(
+                    filename,
+                    strict_node_history=False,
+                    discover_changed_paths=True,
+                    limit=limit,
+                    revision_end=revision_end )
+
+    repos_path = info.URL[len(info.repos_root_URL):]
+    for log in log_entries:
+        history_entries.append(
+            LogEntry( log.revision.number,
+                info.repos_root_URL+repos_path,
+                log.author,
+                log.date,
+                '',
+                log.message,
+                log.changed_paths ) )
+
+        for changed_path in log.changed_paths:
+            if changed_path.action in ['A','M']:
+                if repos_path == changed_path.path:
+                    if changed_path.copyfrom_path is not None:
+                        repos_path = changed_path.copyfrom_path
+                    break
+
+    tags_url = project_info.getTagsUrl( info.URL )
+    if tags_url:
+        for ls_info in project_info.client_bg.ls( tags_url ):
+            history_entries.append( LogEntry(ls_info.created_rev.number,
+                '',
+                ls_info.last_author,
+                ls_info.time,
+                'Tag ' + ls_info.name.split('/')[-1],
+                '',
+                []) )
+
+    history_entries.sort()
+    return info.URL, history_entries
+
 class HistoryDialog(wx.Dialog):
     def __init__( self, parent ):
         wx.Dialog.__init__( self, parent, -1, 'Log History' )
@@ -142,7 +211,7 @@ class HistoryFileFrame(wx.Frame):
     def __init__( self, app, project_info, filename, url, log_entries ):
         wx.Frame.__init__( self, None, -1, "History of %s" % filename, size=(700,500) )
 
-        self.panel = HistoryFilePanel( self, app, project_info, filename, url, log_entries )
+        self.panel = LogHistoryPanel( self, app, project_info, filename, url, log_entries )
 
         # Set the application icon
         self.SetIcon( wb_images.getIcon( 'wb.png' ) )
@@ -152,11 +221,16 @@ class HistoryFileFrame(wx.Frame):
     def OnCloseWindow( self, event ):
         self.Destroy()
 
+    def diffFunction( self, app, project_info, info1, info2 ):
+        return wb_subversion_diff.subversionDiffFiles(
+                    app, project_info,
+                    info1, info2 )
+
 class HistoryDirFrame(wx.Frame):
-    def __init__( self, app, filename, url, log_entries ):
+    def __init__( self, app, project_info, filename, url, log_entries ):
         wx.Frame.__init__( self, None, -1, "History of %s" % filename, size=(700,500) )
 
-        self.panel = HistoryDirPanel( self, app, filename, url, log_entries )
+        self.panel = LogHistoryPanel( self, app, project_info, filename, url, log_entries )
 
         # Set the application icon
         self.SetIcon( wb_images.getIcon( 'wb.png' ) )
@@ -166,8 +240,12 @@ class HistoryDirFrame(wx.Frame):
     def OnCloseWindow( self, event ):
         self.Destroy()
 
+    def diffFunction( self, app, project_info, info1, info2 ):
+        return wb_subversion_diff.subversionDiffDir(
+                    app, project_info,
+                    info1, info2 )
 
-class HistoryCommonPanel:
+class LogHistoryPanel:
     col_revision = 0
     col_author = 1
     col_date = 2
@@ -179,8 +257,10 @@ class HistoryCommonPanel:
     col_copyfrom_revision = 2
     col_copyfrom_path = 3
 
-    def __init__( self, parent, app, filename, url, log_entries ):
+    def __init__( self, parent, app, project_info, filename, url, log_entries ):
+        self.parent = parent
         self.app = app
+        self.project_info = project_info
         self.filename = filename
         self.url = url
         self.log_entries = log_entries
@@ -279,98 +359,7 @@ class HistoryCommonPanel:
                 panel.Layout()
 
     def cmpLogEntries( self, a, b ):
-        return cmp( a[0], b[0] )
-
-    def initButtons( self, sizer ):
-        pass
-
-    #---------- Event handlers ----------------------------------------------------------
-    def OnListItemSelected( self, event):
-        pass
-
-    def OnListItemDeselected( self, event):
-        pass
-
-    def OnPathItemSelected( self, event):
-        pass
-
-    def OnPathItemDeselected( self, event):
-        pass
-
-    #---------- Comment handlers ------------------------------------------------------------
-    def updateComment( self, index ):
-        if index >= 0:
-            message = self.log_entries[ index ][4]
-            all_paths_info = self.log_entries[ index ][5]
-        else:
-            message = ''
-            all_paths_info = []
-
-        self.comment_ctrl.SetValue( message )
-        self.comment_ctrl.SetInsertionPoint( 0 )
-
-        self.log_entries.sort( self.by_rev )
-
-        self.paths_ctrl.DeleteAllItems()
-        all_paths_info.sort( self.by_changed_path )
-        for index, info in enumerate( all_paths_info ):
-            self.paths_ctrl.InsertStringItem( index,
-                action_map.get( info['action'], info['action'] ) )
-            self.paths_ctrl.SetStringItem( index, self.col_path,
-                info['path'] )
-            if info['copyfrom_path'] is not None:
-                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_revision,
-                    str( info['copyfrom_revision'].number ) )
-                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_path,
-                    info['copyfrom_path'] )
-            else:
-                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_revision, '' )
-                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_path, '' )
-
-    def OnSizeHistory( self, event):
-        w,h = self.panel_history.GetClientSizeTuple()
-        self.v_sizer_history.SetDimension( 0, 0, w, h )
-
-    def OnSizeComment( self, event):
-        w,h = self.panel_comment.GetClientSizeTuple()
-        self.v_sizer_comment.SetDimension( 0, 0, w, h )
-
-    def OnSizeChangedPaths( self, event):
-        w,h = self.panel_changed_paths.GetClientSizeTuple()
-        self.v_sizer_changed_paths.SetDimension( 0, 0, w, h )
-
-    def by_changed_path( self, a, b ):
-        return cmp( a['path'], b['path'] )
-
-    def by_rev( self, a, b ):
-        # highest rev first
-        return -cmp( a[0], b[0] )
-
-    def initList( self ):
-        self.log_entries.sort( self.by_rev )
-
-        for index, (rev,author,date,label,message,paths) in enumerate( self.log_entries ):
-            self.list_ctrl.InsertStringItem( index, str(rev) )
-            self.list_ctrl.SetStringItem( index, self.col_author, author )
-            self.list_ctrl.SetStringItem( index, self.col_date, wb_subversion_utils.fmtDateTime( date ) )
-            self.list_ctrl.SetStringItem( index, self.col_label, label )
-            self.list_ctrl.SetStringItem( index, self.col_message, message.replace( '\n', ' ' ) )
-
-class HistoryDirPanel(HistoryCommonPanel):
-    def __init__( self, parent, app, filename, url, log_entries ):
-        HistoryCommonPanel.__init__( self, parent, app, filename, url, log_entries )
-
-    #---------- Event handlers ----------------------------------------------------------
-    def OnListItemSelected( self, event):
-        self.updateComment( event.m_itemIndex )
-
-    def OnListItemDeselected( self, event):
-        self.updateComment( -1 )
-
-class HistoryFilePanel(HistoryCommonPanel):
-    def __init__( self, parent, app, project_info, filename, url, log_entries ):
-        HistoryCommonPanel.__init__( self, parent, app, filename, url, log_entries )
-        self.project_info = project_info
+        return cmp( a.rev_number, b.rev_number )
 
     def initButtons( self, sizer ):
         self.h_sizer = wx.BoxSizer( wx.HORIZONTAL )
@@ -390,62 +379,6 @@ class HistoryFilePanel(HistoryCommonPanel):
         wx.EVT_BUTTON( self.panel_history, id_view_cmd, self.OnViewCommand )
         wx.EVT_BUTTON( self.panel_history, id_diff_cmd, self.OnDiffCommand )
 
-    #---------- Command handlers ----------------------------------------------------------
-    def OnViewCommand( self, event ):
-        print 'Log history View not implemented'
-
-    def OnDiffCommand( self, event ):
-        indices = self.selected_revisions.keys()
-        indices.sort()
-        indices.reverse()
-
-        if len( indices ) > 2:
-            return
-
-        revnum1 = None
-        revnum2 = None
-        rev1_lines = None
-        rev2_lines = None
-
-        if len( indices ) >= 1:
-            index = indices[0]
-            try:
-                revnum1 = self.log_entries[ index ][0]
-                rev1 = pysvn.Revision( pysvn.opt_revision_kind.number, revnum1 )
-
-                rev1_text = self.project_info.client_bg.cat( self.filename, revision=rev1 )
-                rev1_lines = wb_read_file.contentsAsUnicode( rev1_text ).split('\n')
-
-            except pysvn.ClientError, e:
-                self.app.log_client_error( e )
-                return
-
-        if len( indices ) == 2:
-            index = indices[1]
-            try:
-                revnum2 = self.log_entries[ index ][0]
-                rev2 = pysvn.Revision( pysvn.opt_revision_kind.number, revnum2 )
-
-                rev2_text = self.project_info.client_bg.cat( self.filename, revision=rev2 )
-                rev2_lines = wb_read_file.contentsAsUnicode( rev2_text ).split('\n')
-
-            except pysvn.ClientError, e:
-                self.app.log_client_error( e )
-                return
-
-        try:
-            if len( indices ) == 1:
-                self.app.Diff( rev1_lines, '%s@%d' % (self.url, revnum1),
-                        self.filename, self.filename )
-            else:
-                self.app.Diff( rev1_lines, '%s@%d' % (self.url, revnum1),
-                        rev2_lines, '%s@%d' % (self.url, revnum2) )
-
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
-            return
-
-
     #---------- Event handlers ----------------------------------------------------------
     def OnListItemSelected( self, event):
         self.updateComment( event.m_itemIndex )
@@ -460,3 +393,119 @@ class HistoryFilePanel(HistoryCommonPanel):
 
         self.button_diff.Enable( len( self.selected_revisions ) in [1] )
         self.button_diff.Enable( len( self.selected_revisions ) in [1,2] )
+
+    def OnPathItemSelected( self, event):
+        pass
+
+    def OnPathItemDeselected( self, event):
+        pass
+
+    #---------- Comment handlers ------------------------------------------------------------
+    def updateComment( self, index ):
+        if index >= 0:
+            message = self.log_entries[ index ].message
+            all_paths_info = self.log_entries[ index ].changed_paths
+        else:
+            message = ''
+            all_paths_info = []
+
+        self.comment_ctrl.SetValue( message )
+        self.comment_ctrl.SetInsertionPoint( 0 )
+
+        self.log_entries.sort( self.by_rev )
+
+        self.paths_ctrl.DeleteAllItems()
+        all_paths_info.sort( self.by_changed_path )
+        for index, info in enumerate( all_paths_info ):
+            self.paths_ctrl.InsertStringItem( index,
+                action_map.get( info.action, info.action ) )
+            self.paths_ctrl.SetStringItem( index, self.col_path,
+                info.path )
+            if info.copyfrom_path is not None:
+                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_revision,
+                    str( info.copyfrom_revision.number ) )
+                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_path,
+                    info.copyfrom_path )
+            else:
+                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_revision, '' )
+                self.paths_ctrl.SetStringItem( index, self.col_copyfrom_path, '' )
+
+    def OnSizeHistory( self, event):
+        w,h = self.panel_history.GetClientSizeTuple()
+        self.v_sizer_history.SetDimension( 0, 0, w, h )
+
+    def OnSizeComment( self, event):
+        w,h = self.panel_comment.GetClientSizeTuple()
+        self.v_sizer_comment.SetDimension( 0, 0, w, h )
+
+    def OnSizeChangedPaths( self, event):
+        w,h = self.panel_changed_paths.GetClientSizeTuple()
+        self.v_sizer_changed_paths.SetDimension( 0, 0, w, h )
+
+    def by_changed_path( self, a, b ):
+        return cmp( a.path, b.path )
+
+    def by_rev( self, a, b ):
+        # highest rev first
+        return -cmp( a.rev_number, b.rev_number )
+
+    def initList( self ):
+        self.log_entries.sort( self.by_rev )
+
+        for index, log_entry in enumerate( self.log_entries ):
+            self.list_ctrl.InsertStringItem( index, str(log_entry.rev_number) )
+            self.list_ctrl.SetStringItem( index, self.col_author, log_entry.author )
+            self.list_ctrl.SetStringItem( index, self.col_date, wb_subversion_utils.fmtDateTime( log_entry.date ) )
+            self.list_ctrl.SetStringItem( index, self.col_label, log_entry.label )
+            self.list_ctrl.SetStringItem( index, self.col_message, log_entry.message.replace( '\n', ' ' ) )
+
+    #---------- Command handlers ----------------------------------------------------------
+    def OnViewCommand( self, event ):
+        print 'Log history View not implemented'
+
+    def OnDiffCommand( self, event ):
+        indices = self.selected_revisions.keys()
+        indices.sort()
+        indices.reverse()
+
+        if len( indices ) not in (1,2):
+            return
+
+
+        info1 = wb_subversion_diff.PathInfoForDiff()
+
+        info1.path = self.log_entries[ indices[0] ].url
+        info1.revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ indices[0] ].rev_number )
+        info1.peg_path = self.log_entries[ 0 ].url
+        info1.peg_revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ 0 ].rev_number )
+        info1.title = '%s@%d' % (info1.path, info1.revision.number)
+
+        info2 = info1.copy()
+
+        if len( indices ) == 1:
+            info2.path = self.filename
+            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+            info2.peg_path = info2.path
+            info2.peg_revision = info2.revision
+            info2.title = self.filename
+        else:
+            info2.path = self.log_entries[ indices[1] ].url
+            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.number, self.log_entries[ indices[1] ].rev_number )
+            info2.title = '%s@%d' % (info2.path, info2.revision.number)
+
+        generator = self.parent.diffFunction(
+                    self.app,
+                    self.project_info,
+                    info1, info2 )
+
+        #
+        #   history does not need to have the diff code run in the background
+        #   so just step the generator
+        #
+        if type(generator) == types.GeneratorType:
+            while True:
+                try:
+                    generator.next()
+                except StopIteration:
+                    # no problem all done
+                    break
