@@ -108,118 +108,99 @@ class SubversionListHandler(wb_subversion_list_handler_common.SubversionListHand
         self.app.refreshFrame()
 
     def Cmd_File_EditPaste( self, all_rows ):
-        print 'Cmd_File_EditPaste'
         if not self.app.hasPasteData():
             return
 
         paste_data = self.app.getPasteData()
         self.app.clearPasteData()
 
-        if paste_data.isCopy():
-            self.__Cmd_PasteCopy( paste_data, 'Paste Copy' )
-        else:
-            self.__Cmd_PasteMove( paste_data, 'Paste Move' )
+        all_status = []
+        try:
+            for filename in paste_data.getAllFilenames():
+                if os.path.isdir( filename ):
+                    dir_status = self.project_info.client_bg.status( os.path.dirname( filename ), recurse=False )
+                    all_status.extend( [s for s in dir_status if s.path == filename] )
+                else:
+                    all_status.extend( self.project_info.client_bg.status( filename, recurse=False ) )
+        except pysvn.ClientError, e:
+            self.app.log_client_error( e )
+            return
 
+        if paste_data.isCopy():
+            title = 'Paste Copy'
+        else:
+            title = 'Paste Move'
+
+        confirmed, force = self.app.confirmForceAction( title, self.getStatusAndFilenames( all_status ) )
+        if not confirmed:
+            return
+
+        self.app.setProgress( title, 0 )
+
+        for status in all_status:
+            ok = False
+
+            old_filename = status.path
+            basename = os.path.basename( old_filename )
+            new_filename = os.path.join( self.project_info.wc_path, basename )
+
+            if paste_data.isCopy():
+                rename_title = 'Save As'
+            else:
+                rename_title = 'Rename'
+
+            while os.path.exists( new_filename ):
+                new_name, force = self.app.renameFile( rename_title, os.path.basename( old_filename ), None )
+                if new_name is None:
+                    return
+                new_filename = os.path.join( self.project_info.wc_path, new_name )
+
+            self.app.log.info( '%s: From %s' % (title, old_filename) )
+            self.app.log.info( '%s:   To %s' % (title, new_filename) )
+
+            is_controlled = self.isControlled( status )
+
+            yield self.app.backgroundProcess
+
+            try:
+                if paste_data.isCopy():
+                    if os.path.isdir( old_filename ):
+                        if is_controlled:
+                            self.project_info.client_bg.copy( old_filename, new_filename )
+                        else:
+                            raise EnvironmentError( 'TBD - implement copy of folder' )
+                            os.copydirtree( old_filename, new_filename )
+                    else:
+                        self.__copyFile( old_filename, new_filename, is_controlled, status.text_status, status.prop_status )
+
+                    ok = True
+                else:
+                    if os.path.isdir( old_filename ):
+                        if is_controlled:
+                            self.project_info.client_bg.move( old_filename, new_filename, force=force )
+                        else:
+                            os.rename( old_filename, new_filename )
+                    else:
+                        text_status = self.getTextStatus( status )
+                        prop_status = self.getPropStatus( status )
+                        self.__moveFile( old_filename, new_filename, is_controlled, text_status, prop_status )
+
+                    ok = True
+
+            except EnvironmentError, e:
+                self.app.log.error( str(e) )
+
+            except pysvn.ClientError, e:
+                self.app.log_client_error( e )
+
+            yield self.app.foregroundProcess
+
+            if not ok:
+                break
+
+        self.app.clearProgress()
         self.app.refreshFrame()
 
-    def __Cmd_PasteCopy( self, paste_data, title ):
-        all_status = []
-        try:
-            for filename in paste_data.getAllFilenames():
-                if os.path.isdir( filename ):
-                    dir_status = self.project_info.client_fg.status( os.path.dirname( filename ), recurse=False )
-                    all_status.extend( [s for s in dir_status if s.path == filename] )
-                else:
-                    all_status.extend( self.project_info.client_fg.status( filename, recurse=False ) )
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
-            return
-
-        confirmed, force = self.app.confirmForceAction( title, self.getStatusAndFilenames( all_status ) )
-        if not confirmed:
-            return
-
-        try:
-            for status in all_status:
-                old_filename = status.path
-                basename = os.path.basename( old_filename )
-                new_filename = os.path.join( self.project_info.wc_path, basename )
-
-                while os.path.exists( new_filename ):
-                    new_name, force = self.app.renameFile( "Save as", os.path.basename( old_filename ), None )
-                    if new_name is None:
-                        return
-                    new_filename = os.path.join( self.project_info.wc_path, new_name )
-
-                print '%s: From %s' % (title, old_filename)
-                print '%s:   To %s' % (title, new_filename)
-                print 'force',repr(force)
-
-                is_controlled = self.isControlled( status )
-                if os.path.isdir( old_filename ):
-                    if is_controlled:
-                        self.project_info.client_fg.copy( old_filename, new_filename )
-                    else:
-                        raise EnvironmentError( 'TBD - implement copy of folder' )
-                        os.copydirtree( old_filename, new_filename )
-                else:
-                    self.__copyFile( old_filename, new_filename, is_controlled, status.text_status, status.prop_status )
-
-        except EnvironmentError, e:
-            self.app.log.error( str(e) )
-
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
-
-    def __Cmd_PasteMove( self, paste_data, title ):
-        all_status = []
-        try:
-            for filename in paste_data.getAllFilenames():
-                if os.path.isdir( filename ):
-                    dir_status = self.project_info.client_fg.status( os.path.dirname( filename ), recurse=False )
-                    all_status.extend( [s for s in dir_status if s.path == filename] )
-                else:
-                    all_status.extend( self.project_info.client_fg.status( filename, recurse=False ) )
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
-            return
-
-        confirmed, force = self.app.confirmForceAction( title, self.getStatusAndFilenames( all_status ) )
-        if not confirmed:
-            return
-
-        try:
-            for status in all_status:
-                old_filename = status.path
-                basename = os.path.basename( old_filename )
-                new_filename = os.path.join( self.project_info.wc_path, basename )
-
-                while os.path.exists( new_filename ):
-                    new_name, force = self.app.renameFile( "Rename", os.path.basename( old_filename ), None )
-                    if new_name is None:
-                        return
-                    new_filename = os.path.join( self.project_info.wc_path, new_name )
-
-                print '%s: From %s' % (title, old_filename)
-                print '%s:   To %s' % (title, new_filename)
-                print 'force',repr(force)
-
-                is_controlled = self.isControlled( status )
-                if os.path.isdir( old_filename ):
-                    if is_controlled:
-                        self.project_info.client_fg.move( old_filename, new_filename, force=force )
-                    else:
-                        os.rename( old_filename, new_filename )
-                else:
-                    text_status = self.getTextStatus( status )
-                    prop_status = self.getPropStatus( status )
-                    self.__moveFile( old_filename, new_filename, is_controlled, text_status, prop_status )
-
-        except EnvironmentError, e:
-            self.app.log.error( str(e) )
-
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
 
     #------------------------------------------------------------
     def Cmd_File_Add( self, all_rows ):
@@ -292,18 +273,21 @@ class SubversionListHandler(wb_subversion_list_handler_common.SubversionListHand
         for row in all_rows:
             old_filename = self.getConflictOld( row )
             new_filename = self.getConflictNew( row )
+            # qqq diff old_filename@None, new_filename@None
             self.app.Diff( old_filename, old_filename, new_filename, new_filename )
 
     def Cmd_File_DiffOldMine( self, all_rows ):
         for row in all_rows:
             old_filename = self.getConflictOld( row )
             mine_filename = self.getConflictMine( row )
+            # qqq diff old_filename@None, mine_filename@None
             self.app.Diff( old_filename, old_filename, mine_filename, mine_filename )
 
     def Cmd_File_DiffMineNew( self, all_rows ):
         for row in all_rows:
             mine_filename = self.getConflictMine( row )
             new_filename = self.getConflictNew( row )
+            # qqq diff mine_filename@None, new_filename@None
             self.app.Diff( mine_filename, mine_filename, new_filename, new_filename )
 
     # Cmd_File_History = from SubversionListHandlerCommon
