@@ -11,15 +11,12 @@
 
 '''
 import os
-import sys
+import tempfile
 import types
 
 import pysvn
 import wx
 
-import wb_config
-
-import wb_ids
 import wb_subversion_history
 import wb_subversion_annotate
 import wb_list_panel_common
@@ -28,6 +25,7 @@ import wb_subversion_utils
 import wb_read_file
 import wb_subversion_info_dialog
 import wb_subversion_properties_dialog
+import wb_subversion_diff
 
 col_labels = [
         ('Name',        25, 10, 100, wx.LIST_FORMAT_LEFT),
@@ -137,7 +135,7 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
         self.__last_wc_path = None
 
-        self.__branch_revision_number = None
+        self.__branch_origin_revision = None
         self.__branch_url = None
         self.__branch_origin_url = None
 
@@ -150,33 +148,33 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
     def updateStatus( self ):
         self.project_info.updateStatus()
 
-    def getBranchRevisionNumber( self ):
-        if self.__branch_revision_number is None:
+    def getBranchOriginRevision( self ):
+        if self.__branch_origin_revision is None:
             try:
                 log_entries = self.project_info.client_bg.log(self.project_info.url, discover_changed_paths=True)
-                branch_log_entry = log_entries[-1]
-                branch_revision = branch_log_entry['revision']
-                self.__branch_revision_number = str(branch_revision.number)
-                changed_paths = branch_log_entry['changed_paths'][0]
-
-                # If the oldest revision has a copyfrom_path, this is a branch,
-                # so we set __branch_origin_url and __branch_url.
-                if changed_paths['copyfrom_path'] is not None:
-                    base_url = '/'.join(self.project_info.url.split('/')[:4])
-                    self.__branch_origin_url = base_url + changed_paths['copyfrom_path']
-                    self.__branch_url = base_url + changed_paths['path']
             except pysvn.ClientError, e:
                 self.app.log_client_error( e )
-        return self.__branch_revision_number
+                return None
+
+            branch_log_entry = log_entries[-1]
+            changed_paths = branch_log_entry['changed_paths'][0]
+
+            # If the oldest revision has a copyfrom_path, this is a branch,
+            # so we set __branch_origin_url, __branch_url and __branch_origin_revision.
+            if changed_paths['copyfrom_path'] is not None:
+                base_url = '/'.join(self.project_info.url.split('/')[:4])
+                self.__branch_origin_url = base_url + changed_paths['copyfrom_path']
+                self.__branch_url = base_url + changed_paths['path']
+                self.__branch_origin_revision = changed_paths['copyfrom_revision']
+
+        return self.__branch_origin_revision
 
     def getBranchOriginUrl( self ):
-        if self.__branch_revision_number is None:
-            self.getBranchRevisionNumber()
+        self.getBranchOriginRevision()
         return self.__branch_origin_url
 
     def getBranchUrl( self ):
-        if self.__branch_revision_number is None:
-            self.getBranchRevisionNumber()
+        self.getBranchOriginRevision()
         return self.__branch_url
 
     def getProjectInfo( self ):
@@ -227,7 +225,7 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
             selection_state = self.__saveListSelectionState()
         else:
             self.app.log.debug( 'initList - changed wc_path to %r' % self.project_info.wc_path )
-            selection_state = (None,[])
+            selection_state = (None, [])
 
         self.__last_wc_path = self.project_info.wc_path
 
@@ -329,27 +327,27 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         status = self.all_files[ index ]
 
         if column == self.col_name:
-            value = self.__get_NameColumn( index, status, prefix_len )
+            value = self.__get_NameColumn( status, prefix_len )
         elif column == self.col_state:
-            value = self.__get_StateColumn( index, status )
+            value = self.__get_StateColumn( status )
         elif column == self.col_date:
-            value = self.__get_DateColumn( index, status )
+            value = self.__get_DateColumn( status )
         elif column == self.col_revision:
-            value = self.__get_RevisionColumn( index, status )
+            value = self.__get_RevisionColumn( status )
         elif column == self.col_author:
-            value = self.__get_AuthorColumn( index, status )
+            value = self.__get_AuthorColumn( status )
         elif column == self.col_type:
-            value = self.__get_TypeColumn( index, status )
+            value = self.__get_TypeColumn( status )
         elif column == self.col_size:
-            value = self.__get_SizeColumn( index, status )
+            value = self.__get_SizeColumn( status )
         elif column == self.col_mime_type:
-            value = self.__get_MimeTypeColumn( index, self.project_info.getProperty( status.path, 'svn:mime-type' ) )
+            value = self.__get_MimeTypeColumn( self.project_info.getProperty( status.path, 'svn:mime-type' ) )
         elif column == self.col_eol_style:
-            value = self.__get_EolStyleColumn( index, self.project_info.getProperty( status.path, 'svn:eol-style' ) )
+            value = self.__get_EolStyleColumn( self.project_info.getProperty( status.path, 'svn:eol-style' ) )
         elif column == self.col_lock_owner:
-            value = self.__get_LockOwnerColumn( index, status )
+            value = self.__get_LockOwnerColumn( status )
         elif column == self.col_lock_comment:
-            value = self.__get_LockCommentColumn( index, status )
+            value = self.__get_LockCommentColumn( status )
         else:
             value = 'Opss'
         return value
@@ -367,7 +365,7 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
         return self.all_item_attr[ colour ]
 
-    def __get_NameColumn( self, index, status, prefix_len ):
+    def __get_NameColumn( self, status, prefix_len ):
         if status.entry is None:
             is_dir = os.path.isdir( status.path )
         else:
@@ -379,56 +377,57 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
         return filename[prefix_len:]
 
-    def __get_SizeColumn( self, index, status ):
+    def __get_SizeColumn( self, status ):
+        type( status )
         return '%d' % 0
 
-    def __get_EolStyleColumn( self, index, value ):
+    def __get_EolStyleColumn( self, value ):
         if value is None:
             value = ''
 
         return value
 
-    def __get_MimeTypeColumn( self, index, value ):
+    def __get_MimeTypeColumn( self, value ):
         if value is None:
             value = ''
 
         return value
 
-    def __get_StateColumn( self, index, status ):
+    def __get_StateColumn( self, status ):
         return self.statusFormatString( status )
 
-    def __get_TypeColumn( self, index, status ):
+    def __get_TypeColumn( self, status ):
         if status.entry is None:
             return ''
         else:
             return str(status.entry.kind)
 
-    def __get_AuthorColumn( self, index, status ):
+    def __get_AuthorColumn( self, status ):
         if status.entry is None or status.entry.commit_author is None:
             return ''
         else:
             return status.entry.commit_author
 
-    def __get_RevisionColumn( self, index, status ):
+    def __get_RevisionColumn( self, status ):
         if status.entry is None or status.entry.commit_revision.number < 0:
             return ''
         else:
             return str(status.entry.commit_revision.number)
 
-    def __get_DateColumn( self, index, status ):
+    def __get_DateColumn( self, status ):
         if status.entry is None or status.entry.commit_time == 0:
             return ''
         else:
             return wb_subversion_utils.fmtDateTime( status.entry.commit_time )
 
-    def __get_LockCommentColumn( self, index, status ):
+    def __get_LockCommentColumn( self, status ):
         if status.repos_lock is not None:
             comment = status.repos_lock['comment'].replace( '\n', ' ' )
         else:
             comment = status.entry.lock_comment.replace( '\n', ' ' )
         return comment
 
-    def __get_LockOwnerColumn( self, index, status ):
+    def __get_LockOwnerColumn( self, status ):
         if status.repos_lock is not None:
             comment = status.repos_lock['owner']
         else:
@@ -455,11 +454,8 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
         state.conflict = True
         state.file_exists = True
 
-        state.filenames = []
-
         for row in all_rows:
             filename = self.all_files[ row ].path
-            state.filenames.append( filename )
 
             if not os.path.exists( filename ):
                 state.file_exists = False
@@ -564,6 +560,9 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
     def statusFormatString( self, status ):
         return wb_subversion_utils._status_format( status )
 
+    def getAllGreyFilenames( self ):
+        raise NotImplemented
+
     def statusColour( self, status ):
         # default colour when nothing special is know for the file
         colour = wx.BLACK
@@ -618,13 +617,33 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
     def Cmd_File_DiffWorkBase( self, all_rows ):
         for filename in [self.getFilename( row ) for row in all_rows]:
             self.app.setAction( 'Diff BASE %s...' % filename )
-            base_filename = os.path.join(
-                self.project_info.wc_path,
-                os.path.dirname( filename ),
-                '.svn', 'text-base',
-                os.path.basename( filename ) + '.svn-base' )
 
-            self.app.Diff( base_filename, filename + '@BASE', filename, filename )
+            info1 = wb_subversion_diff.PathInfoForDiff()
+
+            info1.path = filename
+            info1.revision = pysvn.Revision( pysvn.opt_revision_kind.base )
+            info1.title = filename + '@BASE'
+
+            info2 = wb_subversion_diff.PathInfoForDiff()
+
+            info2.path = filename
+            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+            info2.title = filename
+
+            generator = wb_subversion_diff.subversionDiffFiles(
+                                self.app,
+                                self.project_info,
+                                info1,
+                                info2 )
+            if type(generator) == types.GeneratorType:
+                while True:
+                    try:
+                        where_to_go_next = generator.next()
+                    except StopIteration:
+                        # no problem all done
+                        break
+
+                    yield where_to_go_next
 
         self.app.setAction( 'Ready' )
 
@@ -634,31 +653,39 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
             self.app.setAction( 'Diff HEAD %s...' % filename )
 
-            yield self.app.backgroundProcess
+            info1 = wb_subversion_diff.PathInfoForDiff()
 
-            ok = False
-            try:
-                head_rev_text = self.project_info.client_bg.cat( filename,
-                            revision=pysvn.Revision( pysvn.opt_revision_kind.head ) )
-                ok = True
-            except pysvn.ClientError, e:
-                self.app.log_client_error( e )
+            info1.path = filename
+            info1.revision = pysvn.Revision( pysvn.opt_revision_kind.head )
+            info1.title = filename + '@HEAD'
 
-            yield self.app.foregroundProcess
+            info2 = wb_subversion_diff.PathInfoForDiff()
 
-            if not ok:
-                break
+            info2.path = filename
+            info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+            info2.title = filename
 
-            head_rev_lines = wb_read_file.contentsAsUnicode( head_rev_text ).split('\n')
-            if head_rev_lines[-1] == '':
-                del head_rev_lines[-1]
-            self.app.Diff( head_rev_lines, url + '@HEAD', filename, filename )
+            generator = wb_subversion_diff.subversionDiffFiles(
+                                self.app,
+                                self.project_info,
+                                info1,
+                                info2 )
+                            
+            if type(generator) == types.GeneratorType:
+                while True:
+                    try:
+                        where_to_go_next = generator.next()
+                    except StopIteration:
+                        # no problem all done
+                        break
+
+                    yield where_to_go_next
 
         self.app.setAction( 'Ready' )
 
     def Cmd_File_DiffWorkBranchOriginBase( self, all_rows ):
-
         self.app.setAction( "Retrieving branch info..." )
+
         yield self.app.backgroundProcess
         branch_url = self.getBranchUrl()
         yield self.app.foregroundProcess
@@ -674,23 +701,33 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
                 self.app.setAction( 'Diff branch origin BASE %s...' % filename )
 
-                yield self.app.backgroundProcess
+                info1 = wb_subversion_diff.PathInfoForDiff()
 
-                ok = False
-                try:
-                    branch_origin_base_rev_text = self.project_info.client_bg.cat( origin_url,
-                        revision=pysvn.Revision( pysvn.opt_revision_kind.number, self.getBranchRevisionNumber() ) )
-                    ok = True
-                except pysvn.ClientError, e:
-                    self.app.log_client_error( e )
+                info1.path = origin_url
+                info1.revision = self.getBranchOriginRevision()
+                info1.title = '%s@%d' % (filename, self.getBranchOriginRevision().number)
 
-                yield self.app.foregroundProcess
+                info2 = wb_subversion_diff.PathInfoForDiff()
 
-                if not ok:
-                    break
+                info2.path = filename
+                info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+                info2.title = filename
 
-                branch_origin_base_rev_lines = wb_read_file.contentsAsUnicode( branch_origin_base_rev_text ).split('\n')
-                self.app.Diff( branch_origin_base_rev_lines, origin_url + '@' + self.getBranchRevisionNumber(), filename, filename )
+                generator = wb_subversion_diff.subversionDiffFiles(
+                                    self.app,
+                                    self.project_info,
+                                    info1,
+                                    info2 )
+
+            if type(generator) == types.GeneratorType:
+                while True:
+                    try:
+                        where_to_go_next = generator.next()
+                    except StopIteration:
+                        # no problem all done
+                        break
+
+                    yield where_to_go_next
 
         self.app.setAction( 'Ready' )
 
@@ -711,23 +748,33 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
 
                 self.app.setAction( 'Diff branch origin HEAD %s...' % filename )
 
-                yield self.app.backgroundProcess
+                info1 = wb_subversion_diff.PathInfoForDiff()
 
-                ok = False
-                try:
-                    branch_origin_head_rev_text = self.project_info.client_bg.cat( origin_url,
-                        revision=pysvn.Revision( pysvn.opt_revision_kind.head ) )
-                    ok = True
-                except pysvn.ClientError, e:
-                    self.app.log_client_error( e )
+                info1.path = origin_url
+                info1.revision = pysvn.Revision( pysvn.opt_revision_kind.head )
+                info1.title = '%s@HEAD' % filename
 
-                yield self.app.foregroundProcess
+                info2 = wb_subversion_diff.PathInfoForDiff()
 
-                if not ok:
-                    break
+                info2.path = filename
+                info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+                info2.title = filename
 
-                branch_origin_head_rev_lines = wb_read_file.contentsAsUnicode( branch_origin_head_rev_text ).split('\n')
-                self.app.Diff( branch_origin_head_rev_lines, origin_url + "@HEAD", filename, filename )
+                generator = wb_subversion_diff.subversionDiffFiles(
+                                    self.app,
+                                    self.project_info,
+                                    info1,
+                                    info2 )
+
+                if type(generator) == types.GeneratorType:
+                    while True:
+                        try:
+                            where_to_go_next = generator.next()
+                        except StopIteration:
+                            # no problem all done
+                            break
+
+                        yield where_to_go_next
 
         self.app.setAction( 'Ready' )
 
@@ -743,35 +790,12 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
             yield self.app.backgroundProcess
 
             ok = False
-            history_entries = []
             try:
-                log_entries = self.project_info.client_bg.log(
-                                filename,
-                                strict_node_history=False,
-                                discover_changed_paths=True,
-                                limit=dialog.getLimit(),
-                                revision_end=dialog.getRevisionEnd() )
-                for log in log_entries:
-                    history_entries.append( (log['revision'].number,
-                        log['author'],
-                        log['date'],
-                        '',
-                        log['message'],
-                        log['changed_paths']) )
-                info = self.project_info.client_bg.info( filename )
-
-                tags_url = self.project_info.getTagsUrl( info.url )
-                if tags_url:
-                    for ls_info in self.project_info.client_bg.ls( tags_url ):
-                        history_entries.append( (ls_info['created_rev'].number,
-                            ls_info['last_author'],
-                            ls_info['time'],
-                            'Tag ' + ls_info['name'].split('/')[-1],
-                            '',
-                            []) )
-
-                history_entries.sort()
-
+                file_url, history_entries = wb_subversion_history.getHistoryEntries(
+                            self.project_info,
+                            filename,
+                            dialog.getLimit(),
+                            dialog.getRevisionEnd() )
                 ok = True
             except pysvn.ClientError, e:
                 self.app.log_client_error( e )
@@ -781,8 +805,12 @@ class SubversionListHandlerCommon(wb_list_panel_common.ListHandler):
             if not ok:
                 break
 
-            h_frame = wb_subversion_history.HistoryFileFrame( self.app, self.project_info,
-                        filename, info.url, history_entries )
+            h_frame = wb_subversion_history.HistoryFileFrame(
+                        self.app,
+                        self.project_info,
+                        filename,
+                        file_url,
+                        history_entries )
             h_frame.Show( True )
 
         self.app.setAction( 'Ready' )
