@@ -11,13 +11,18 @@
 
 '''
 import os
+import types
+
 import pysvn
+
 import wx
+
 import wb_tree_panel
 import wb_ids
 import wb_subversion_utils
 import wb_subversion_project_info
 import wb_subversion_utils
+import wb_subversion_diff
 import wb_subversion_info_dialog
 import wb_subversion_history
 import wb_subversion_checkin
@@ -164,6 +169,9 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         menu_item =[('', wb_ids.id_Command_Shell, '&Command Shell')
             ,('', wb_ids.id_File_Browser, '&File Browser')
             ,('-', 0, 0 )
+            ,('', wb_ids.id_SP_DiffWorkBase, 'Diff WC vs. BASE...' )
+            ,('', wb_ids.id_SP_DiffWorkHead, 'Diff WC vs. HEAD...' )
+            ,('-', 0, 0 )
             ,('', wb_ids.id_SP_History, 'Log history...' )
             ,('', wb_ids.id_SP_Info, 'Information...' )
             ,('', wb_ids.id_SP_Properties, 'Properties...' )
@@ -199,8 +207,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         self.app.refreshFrame()
 
     def Cmd_Dir_EditPaste( self ):
-        print 'Cmd_Dir_EditPaste'
-        self.app.frame.list_panel.OnSpEditPaste()
+        return self.app.frame.list_panel.OnSpEditPaste()
 
     def Cmd_Dir_Add( self ):
         name = os.path.basename( self.project_info.wc_path )
@@ -300,6 +307,78 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
                 self.app.log_client_error( e )
             self.app.refreshFrame()
 
+    def Cmd_Dir_DiffWorkBase( self ):
+        filename = self.project_info.wc_path
+
+        self.app.setAction( 'Diff BASE %s...' % filename )
+
+        info1 = wb_subversion_diff.PathInfoForDiff()
+
+        info1.path = filename
+        info1.revision = pysvn.Revision( pysvn.opt_revision_kind.base )
+        info1.title = filename + '@BASE'
+
+        info2 = wb_subversion_diff.PathInfoForDiff()
+
+        info2.path = filename
+        info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+        info2.title = filename
+
+        generator = wb_subversion_diff.subversionDiffDir(
+                        self.app,
+                        self.project_info,
+                        info1,
+                        info2 )
+
+        if type(generator) == types.GeneratorType:
+            while True:
+                try:
+                    where_to_go_next = generator.next()
+                    
+                except StopIteration:
+                    # no problem all done
+                    break
+
+                yield where_to_go_next
+
+        self.app.setAction( 'Ready' )
+
+    def Cmd_Dir_DiffWorkHead( self ):
+        filename = self.project_info.wc_path
+
+        self.app.setAction( 'Diff HEAD %s...' % filename )
+
+
+        info1 = wb_subversion_diff.PathInfoForDiff()
+
+        info1.path = filename
+        info1.revision = pysvn.Revision( pysvn.opt_revision_kind.head )
+        info1.title = filename + '@HEAD'
+
+        info2 = wb_subversion_diff.PathInfoForDiff()
+
+        info2.path = filename
+        info2.revision = pysvn.Revision( pysvn.opt_revision_kind.working )
+        info2.title = filename
+
+        generator = wb_subversion_diff.subversionDiffDir(
+                            self.app,
+                            self.project_info,
+                            info1,  info2 )
+
+        if type(generator) == types.GeneratorType:
+            while True:
+                try:
+                    where_to_go_next = generator.next()
+                except StopIteration:
+                    # no problem all done
+                    break
+
+                yield where_to_go_next
+
+        self.app.setAction( 'Ready' )
+
+
     def Cmd_Dir_History( self ):
         filename = self.project_info.wc_path
 
@@ -315,34 +394,11 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         ok = False
         history_entries = []
         try:
-            log_entries = self.project_info.client_bg.log(
-                            filename,
-                            strict_node_history=False,
-                            discover_changed_paths=True,
-                            limit=dialog.getLimit(),
-                            revision_end=dialog.getRevisionEnd() )
-            for log in log_entries:
-                history_entries.append( (log['revision'].number,
-                    log['author'],
-                    log['date'],
-                    '',
-                    log['message'],
-                    log['changed_paths']) )
-            info = self.project_info.client_bg.info( filename )
-
-            tags_url = self.project_info.getTagsUrl( info.url )
-            if tags_url:
-                for ls_info in self.project_info.client_bg.ls( tags_url ):
-                    history_entries.append( (ls_info['created_rev'].number,
-                        ls_info['last_author'],
-                        ls_info['time'],
-                        'Tag ' + ls_info['name'].split('/')[-1],
-                        '',
-                        [] ) )
-
-
-            history_entries.sort()
-
+            file_url, history_entries = wb_subversion_history.getHistoryEntries(
+                        self.project_info,
+                        filename,
+                        dialog.getLimit(),
+                        dialog.getRevisionEnd() )
             ok = True
         except pysvn.ClientError, e:
             self.app.log_client_error( e )
@@ -350,7 +406,12 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         yield self.app.foregroundProcess
 
         if ok:
-            h_frame = wb_subversion_history.HistoryDirFrame( self.app, filename, info.url, history_entries )
+            h_frame = wb_subversion_history.HistoryDirFrame(
+                self.app,
+                self.project_info,
+                filename,
+                file_url,
+                history_entries )
             h_frame.Show( True )
 
         self.app.setAction( 'Ready' )
@@ -358,12 +419,15 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
 
     def Cmd_Dir_Info( self ):
         try:
-            entry = self.project_info.client_fg.info( self.project_info.wc_path )
+            if hasattr( self.project_info.client_fg, 'info2' ):
+                entry = self.project_info.client_fg.info2( filename, recurse=False )
+            else:
+                entry = self.project_info.client_fg.info( filename )
 
             dialog = wb_subversion_info_dialog.InfoDialog( self.app,
                     self.app.frame.tree_panel.tree_ctrl,
                     self.project_info.wc_path,
-                    entry )
+                    entry[0] )
             dialog.ShowModal()
 
         except pysvn.ClientError, e:
@@ -396,14 +460,15 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         dialog = wb_dialogs.NewFile( self.app.frame.tree_panel.tree_ctrl, template_dir )
         if dialog.ShowModal() == wx.ID_OK:
             template_contents = ''
-            if template_dir != '':
+            if( template_dir != ''
+            and dialog.getTemplateFilename() is not None ):
                 try:
                     template_filename = os.path.join( template_dir, dialog.getTemplateFilename() )
                     t = file( template_filename, 'r' )
                     template_contents = t.read()
                     t.close()
                 except EnvironmentError, e:
-                    self.log.error( 'Cannot read template %s - %s' % (template_filename, str(e)) )
+                    self.app.log.error( 'Cannot read template %s - %s' % (template_filename, str(e)) )
                     return
 
             try:
@@ -412,7 +477,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
                 f.write( template_contents )
                 f.close()
             except EnvironmentError, e:
-                self.log.error( 'Cannot create new file %s - %s' % (new_filename, str(e)) )
+                self.app.log.error( 'Cannot create new file %s - %s' % (new_filename, str(e)) )
                 return
 
             try:
@@ -534,8 +599,8 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             ci_frame = wb_subversion_report_updates.ReportUpdatesFrame( self.app, self.project_info, all_files )
             ci_frame.Show( True )
 
-    def Cmd_Dir_ReportBranchChanges(self):
-        self.app.setAction('Branch changes %s...' % self.project_info.url)
+    def Cmd_Dir_ReportBranchChanges( self ):
+        self.app.setAction( 'Branch changes %s...' % self.project_info.url )
 
         changed_files = []
         is_branch = False
@@ -543,38 +608,38 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         yield self.app.backgroundProcess
         
         try:
-            log_entries = self.project_info.client_bg.log(self.project_info.url, discover_changed_paths=True)
+            log_entries = self.project_info.client_bg.log( self.project_info.url, discover_changed_paths=True )
             if log_entries[-1]['changed_paths'][0]['copyfrom_path'] is not None:
-               is_branch = True
-               all_files = self.project_info.client_bg.status(self.project_info.wc_path,
-                   recurse=True, get_all=True, ignore=True, update=False)
-               repos_root_url = self.project_info.client_bg.info2(self.project_info.wc_path)[0][1]['repos_root_URL']
-               changed_urls = []
-               for log_entry in log_entries:
-                   for changed_paths in log_entry['changed_paths']:
-                       url = repos_root_url + changed_paths['path']
-                       if not (url in changed_urls):
-                          changed_urls.append(url)
-               for status in all_files:
-                   if (status.entry is not None 
-                       and status.entry.kind == pysvn.node_kind.file
-                       and (status.text_status != pysvn.wc_status_kind.normal or 
-                            status.entry.url in changed_urls)):
-                       changed_files.append(status)
+                is_branch = True
+                all_files = self.project_info.client_bg.status( self.project_info.wc_path,
+                    recurse=True, get_all=True, ignore=True, update=False )
+                repos_root_url = self.project_info.client_bg.info2( self.project_info.wc_path )[0][1]['repos_root_URL']
+                changed_urls = []
+                for log_entry in log_entries:
+                    for changed_paths in log_entry['changed_paths']:
+                        url = repos_root_url + changed_paths['path']
+                        if not (url in changed_urls):
+                            changed_urls.append( url )
+                for status in all_files:
+                    if (status.entry is not None 
+                        and status.entry.kind == pysvn.node_kind.file
+                        and (status.text_status != pysvn.wc_status_kind.normal or 
+                             status.entry.url in changed_urls)):
+                        changed_files.append( status )
         except pysvn.ClientError, e:
-            self.app.log_client_error(e)
+            self.app.log_client_error( e )
 
         yield self.app.foregroundProcess
 
         if not is_branch:
-            wx.MessageBox('"%s" is not a branch.' % self.project_info.url, "Error", style=wx.OK|wx.ICON_ERROR)
-        elif len(changed_files) == 0:
-            wx.MessageBox('No files changed yet in branch "%s".', "Error", style=wx.OK|wx.ICON_ERROR)
+            wx.MessageBox( '"%s" is not a branch.' % self.project_info.url, "Error", style=wx.OK|wx.ICON_ERROR )
+        elif len( changed_files ) == 0:
+            wx.MessageBox( 'No files changed yet in branch "%s".' % self.project_info.url, "Error", style=wx.OK|wx.ICON_ERROR )
         else:
-            ci_frame = wb_subversion_report_branch_changes.ReportBranchChangesFrame(self.app, self.project_info, changed_files)
-            ci_frame.Show(True)
+            ci_frame = wb_subversion_report_branch_changes.ReportBranchChangesFrame( self.app, self.project_info, changed_files )
+            ci_frame.Show( True )
 
-        self.app.setAction('Ready')
+        self.app.setAction( 'Ready' )
 
 
     def Cmd_Dir_Rename( self ):
