@@ -13,6 +13,7 @@
 import wx
 import cPickle
 import os
+import pysvn
 
 import wb_exceptions
 import wb_source_control_providers
@@ -27,6 +28,7 @@ class TreeState:
     def __init__( self ):
         self.modified = False
         self.versioned = False
+        self.new_versioned = False
         self.unversioned = False
         self.need_checkin = False
         self.need_checkout = False
@@ -35,7 +37,7 @@ class TreeState:
         self.is_project_parent = False
 
     def printState( self, title='' ):
-        print '--------',title
+        print '--------', title
         for item in self.__dict__.items():
             print 'TreeState: %s -> %r' % item
 
@@ -64,7 +66,7 @@ class WbTreePanel(wx.Panel):
         if wx.Platform == '__WXMAC__':
             acc_mod = wx.ACCEL_ALT
 
-            acc_init =[
+            acc_init = [
                 (wx.ACCEL_ALT, ord('C'), wb_ids.id_SP_EditCopy),
                 (wx.ACCEL_ALT, ord('X'), wb_ids.id_SP_EditCut),
                 (wx.ACCEL_ALT, ord('V'), wb_ids.id_SP_EditPaste),
@@ -79,7 +81,7 @@ class WbTreePanel(wx.Panel):
                 (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
                 ]
         elif wx.Platform == '__WXMSW__':
-            acc_init =[
+            acc_init = [
                 (wx.ACCEL_CTRL, ord('C'), wb_ids.id_SP_EditCopy),
                 (wx.ACCEL_CTRL, ord('X'), wb_ids.id_SP_EditCut),
                 (wx.ACCEL_CTRL, ord('V'), wb_ids.id_SP_EditPaste),
@@ -96,7 +98,7 @@ class WbTreePanel(wx.Panel):
             acc_init.append( (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open) )
         else:
             # Unix
-            acc_init =[
+            acc_init = [
                 (wx.ACCEL_CTRL, ord('C'), wb_ids.id_SP_EditCopy),
                 (wx.ACCEL_CTRL, ord('X'), wb_ids.id_SP_EditCut),
                 (wx.ACCEL_CTRL, ord('V'), wb_ids.id_SP_EditPaste),
@@ -128,7 +130,7 @@ class WbTreePanel(wx.Panel):
         wx.EVT_MENU( self, wb_ids.id_Project_Delete, try_wrapper( self.OnProjectDelete ) )
         wx.EVT_MENU( self, wb_ids.id_Project_Update, try_wrapper( self.OnProjectUpdate ) )
 
-        if wx.Platform in ['__WXMSW__','__WXMAC__']:
+        if wx.Platform in ['__WXMSW__', '__WXMAC__']:
             wx.EVT_TREE_BEGIN_DRAG( self.tree_ctrl, self.tree_ctrl.GetId(), self.app.eventWrapper( self.OnDragBegin ) )
             wx.EVT_TREE_END_DRAG( self.tree_ctrl, self.tree_ctrl.GetId(), self.app.eventWrapper( self.OnDragEnd ) )
 
@@ -142,8 +144,40 @@ class WbTreePanel(wx.Panel):
         return '<WbTreePanel %r>' % self.getSelectionProjectInfo()
 
     def initFrame( self ):
-        self.initTree( self.app.prefs.getProjects().getProjectList() )
-        self.gotoBookmark( self.last_position_bookmark_name )
+        bookmark_pi = None
+
+        project_list = self.app.prefs.getProjects().getProjectList()
+
+        if( self.app.auto_project_dir is not None
+        and wb_source_control_providers.hasProvider( 'subversion' ) ):
+            provider = wb_source_control_providers.getProvider( 'subversion' )
+            cmd_project_info = provider.getProjectInfo( self.app )
+            project_name = os.path.basename( self.app.auto_project_dir )
+
+            try:
+                url = pysvn.Client().info( self.app.auto_project_dir ).url
+
+                cmd_project_info.init( project_name, url = url, wc_path = self.app.auto_project_dir )
+
+                for project in project_list:
+                    if project.isChild( cmd_project_info ):
+                        bookmark_pi = cmd_project_info
+                        break
+                else:
+                    self.app.prefs.getProjects().addProject( cmd_project_info )
+                    project_list = self.app.prefs.getProjects().getProjectList()
+
+                    bookmark_pi = cmd_project_info
+
+            except pysvn.ClientError:
+                pass
+
+        self.initTree( project_list )
+
+        if bookmark_pi is not None:
+            self.gotoProjectInfo( bookmark_pi )
+        else:
+            self.gotoBookmark( self.last_position_bookmark_name )
 
     def OnSetFocus( self, event ):
         if wb_config.debug_selection: print 'Z: WbTreePanel OnSetFocus'
@@ -394,7 +428,7 @@ class WbTreePanel(wx.Panel):
 
     # Handler for when a new selection is made in the tree control child
     def OnTreeSelChanged( self, event ):
-        if wb_config.debug_selection: print 'Z: WbTreePanel OnTreeSelChanged __skip_update_ui',self.__skip_update_ui
+        if wb_config.debug_selection: print 'Z: WbTreePanel OnTreeSelChanged __skip_update_ui', self.__skip_update_ui
         if self.__skip_update_ui:
             return
 
@@ -557,7 +591,7 @@ class WbTreePanel(wx.Panel):
             return
 
         dialog = wb_dialogs.ConfirmAction( self, 'Delete Project',
-                [('',tree_pi.project_name)] )
+                [('', tree_pi.project_name)] )
 
         rc = dialog.ShowModal()
         if rc == wx.ID_OK:
@@ -657,6 +691,12 @@ class WbTreePanel(wx.Panel):
     def OnSpDelete( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Delete' )
 
+    def OnSpDiffWorkBase( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_DiffWorkBase' )
+
+    def OnSpDiffWorkHead( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_DiffWorkHead' )
+ 
     def OnSpHistory( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_History' )
 
@@ -714,7 +754,7 @@ class WbTreePanel(wx.Panel):
         self.app.trace.info( 'WbTreePanel.Sp_Dispatch( %s ) calling' % sp_func_name )
         fn = getattr( handler, sp_func_name, None )
         if fn is None:
-            print 'Not implemented',sp_func_name
+            print 'Not implemented', sp_func_name
             return None
         else:
             return fn()    
@@ -733,8 +773,8 @@ class WbTreePanel(wx.Panel):
         return getattr( handler, sp_func_name )( filename_list )
 
 class SvnDropTarget(wx.PyDropTarget):
-    def __init__(self, window):
-        wx.PyDropTarget.__init__(self)
+    def __init__( self, window ):
+        wx.PyDropTarget.__init__( self )
         self.tree = window
 
         # specify the type of data we will accept
@@ -742,61 +782,60 @@ class SvnDropTarget(wx.PyDropTarget):
         self.data = wx.CustomDataObject(self.df)
         self.SetDataObject(self.data)
 
-	self.start_item = None
+        self.start_item = None
 
     # some virtual methods that track the progress of the drag
-    def OnEnter(self, x, y, d):
+    def OnEnter( self, x, y, d ):
         #print 'OnEnter: %d, %d, %d' % (x, y, d)
         return d
 
-    def OnLeave(self):
+    def OnLeave( self ):
         #print 'OnLeave'
-	if self.start_item is not None:
-		#print 'OnLeave: have start_item'
-		if self.start_item != self.tree.tree_ctrl.GetSelection():
-			#print 'OnLeave: restoring start_item'
-			self.tree.tree_ctrl.SelectItem( self.start_item )
-			self.tree.tree_ctrl.EnsureVisible( self.start_item )
-		self.start_item = None
+        if self.start_item is not None:
+            #print 'OnLeave: have start_item'
+            if self.start_item != self.tree.tree_ctrl.GetSelection():
+                #print 'OnLeave: restoring start_item'
+                self.tree.tree_ctrl.SelectItem( self.start_item )
+                self.tree.tree_ctrl.EnsureVisible( self.start_item )
+            self.start_item = None
 
-    def OnDrop(self, x, y):
+    def OnDrop( self, x, y ):
         #print 'OnDrop: %d %d' % (x, y)
-	item, flags = self.tree.tree_ctrl.HitTest( (x,y) )
+        item, flags = self.tree.tree_ctrl.HitTest( (x, y) )
 
-	if (flags & wx.TREE_HITTEST_ONITEMLABEL) == 0:
-		return False
+        if (flags & wx.TREE_HITTEST_ONITEMLABEL) == 0:
+            return False
 
         return True
 
-    def OnDragOver(self, x, y, d):
+    def OnDragOver( self, x, y, d ):
         #print 'OnDragOver: %d, %d, %d' % (x, y, d)
-	item, flags = self.tree.tree_ctrl.HitTest( (x,y) )
+        item, flags = self.tree.tree_ctrl.HitTest( (x, y) )
 
-	if self.start_item is None:
-		self.start_item = self.tree.tree_ctrl.GetSelection()
+        if self.start_item is None:
+            self.start_item = self.tree.tree_ctrl.GetSelection()
 
-	if flags & wx.TREE_HITTEST_ONITEMLABEL:
+        if flags & wx.TREE_HITTEST_ONITEMLABEL:
+            if item != self.tree.tree_ctrl.GetSelection():
+                self.tree.tree_ctrl.SelectItem( item )
+                self.tree.tree_ctrl.EnsureVisible( item )
+            return d
 
-		if item != self.tree.tree_ctrl.GetSelection():
-			self.tree.tree_ctrl.SelectItem( item )
-			self.tree.tree_ctrl.EnsureVisible( item )
-		return d
-
-	return wx.DragNone
+        return wx.DragNone
 
     # Called when OnDrop returns True.  We need to get the data and
     # do something with it.
-    def OnData(self, x, y, d):
+    def OnData( self, x, y, d ):
         #print 'OnData: %d, %d, %d' % (x, y, d)
 
         # copy the data from the drag source to our data object
         if self.GetData():
             # convert it back to a list of lines and give it to the viewer
             all_filenames = cPickle.loads( self.data.GetData() )
-	    if d == wx.DragCopy:
-		self.tree.OnSpCopy( all_filenames )
-	    elif d == wx.DragMove:
-		self.tree.OnSpMove( all_filenames )
+            if d == wx.DragCopy:
+                self.tree.OnSpCopy( all_filenames )
+            elif d == wx.DragMove:
+                self.tree.OnSpMove( all_filenames )
             
         # what is returned signals the source what to do
         # with the original data (move, copy, etc.)  In this
@@ -823,16 +862,16 @@ class WbTreeCtrl(wx.TreeCtrl):
             style = wx.TR_HAS_BUTTONS|wx.NO_BORDER|wx.TR_HIDE_ROOT
  
         wx.TreeCtrl.__init__( self, parent, self.id_tree, 
-                wx.DefaultPosition,
-                wx.DefaultSize,
-                style )
+                                wx.DefaultPosition,
+                                wx.DefaultSize,
+                                style )
 
         self.item_last_used = None
 
         try_wrapper = wb_exceptions.TryWrapperFactory( self.app.log )
 
     # Returns the window identifier for this control
-    def getId(self):
+    def getId( self ):
         return self.id_tree
 
     # Searches the child items of a tree item for the child that matches the
